/*
---

name: Bootstrap.Tooltip

description: A simple tooltip implementation that works with the Twitter Bootstrap css framework.

authors: [Aaron Newton]

license: MIT-style license.

requires:
 - /Bootstrap
 - /CSSEvents
 - More/Element.Position
 - More/Element.Shortcuts
 - Behavior/Behavior

provides: [Bootstrap.Twipsy, Bootstrap.Tooltip]

...
*/

Bootstrap.Tooltip = Bootstrap.Twipsy = new Class({

	Implements: [Options, Events],

	options: {
		location: 'above', //below, left, right, bottom, top
		animate: true,
		delayIn: 200,
		delayOut: 0,
		fallback: '',
		override: '',
		onOverflow: false,
		offset: 0,
		title: 'title', //element property
		trigger: 'hover', //focus, manual
		getContent: function(el){
			return el.get(this.options.title);
		},
		inject: {
			target: null, //defaults to document.body,
			where: 'bottom'
		}
	},

	initialize: function(el, options){
		this.element = document.id(el);
		this.setOptions(options);
		var location = this.options.location;
		if (location == 'above') this.options.location = 'top';    //bootstrap 2.0
		if (location == 'below') this.options.location = 'bottom'; //bootstrap 2.0
		this._attach();
	},

	show: function(){
		this._clear();
		this._makeTip();
		var pos, edge, offset = {x: 0, y: 0};
		switch(this.options.location){
			case 'below': case 'bottom':
				pos = 'centerBottom';
				edge = 'centerTop';
				offset.y = this.options.offset;
				break;
			case 'left':
				pos = 'centerLeft';
				edge = 'centerRight';
				offset.x = this.options.offset;
				break;
			case 'right':
				pos = 'centerRight';
				edge = 'centerLeft';
				offset.x = this.options.offset;
				break;
			default: //top
				pos = 'centerTop';
				edge = 'centerBottom';
				offset.y = this.options.offset;
		}
		if (typeOf(this.options.offset) == "object") offset = this.options.offset;
		if (this.element.getParent('.modal')) this.tip.inject(this.element, 'after');
		else this.tip.inject(this.options.inject.target || document.body, this.options.inject.where);
		this.tip.show().position({
			relativeTo: this.element,
			position: pos,
			edge: edge,
			offset: offset
		}).removeClass('out').addClass('in');
		this.visible = true;
		if (!Browser.Features.cssTransition || !this.options.animate) this._complete();
		this.fireEvent('show');
		return this;
	},

	hide: function(){
		this._makeTip();
		this.tip.removeClass('in').addClass('out');
		this.visible = false;
		if (!Browser.Features.cssTransition || !this.options.animate) this._complete();
		this.fireEvent('hide');
		return this;
	},

	destroy: function(){
		this._detach();
		if (this.tip) this.tip.destroy();
		this.destroyed = true;
		return this;
	},

	toggle: function(){
		return this[this.visible ? 'hide' : 'show']();
	},

	// PRIVATE METHODS

	_makeTip: function(){
		if (!this.tip){
			var location = this.options.location;
			if (location == 'above') location = 'top';    //bootstrap 2.0
			if (location == 'below') location = 'bottom'; //bootstrap 2.0
			this.tip = new Element('div.tooltip').addClass(location)
				 .adopt(new Element('div.tooltip-arrow'))
				 .adopt(
				   new Element('div.tooltip-inner', {
				     html: this.options.override || this.options.getContent.apply(this, [this.element]) || this.options.fallback
				   })
				 );
			if (this.options.animate) this.tip.addClass('fade');
			if (Browser.Features.cssTransition && this.tip.addEventListener){
				this.tip.addEventListener(Browser.Features.transitionEnd, this.bound.complete);
			}
			this.element.set('alt', '').set('title', '');
		}
		return this.tip;
	},

	_attach: function(method){
		method = method || 'addEvents';
		if ( ! this.bound) this.bound = {
			enter: this._enter.bind(this),
			leave: this._leave.bind(this),
			complete: this._complete.bind(this),
			toggle: this.toggle.bind(this)
		};

		if (this.options.trigger == 'hover') {
			this.element[method]({
				mouseenter: this.bound.enter,
				mouseleave: this.bound.leave
			});
		} else if (this.options.trigger == 'focus'){
			this.element[method]({
				focus: this.bound.enter,
				blur: this.bound.leave
			});
		} else if (this.options.trigger == 'click'){
			this.element[method]({
				click: this.bound.toggle
			});
		}
	},

	_detach: function(){
		this._attach('removeEvents');
	},

	_clear: function(){
		clearTimeout(this._inDelay);
		clearTimeout(this._outDelay);
	},

	_enter: function(){
		if (this.options.onOverflow){
			var scroll = this.element.getScrollSize(),
			    size = this.element.getSize();
			if (scroll.x <= size.x && scroll.y <= size.y) return;
		}
		this._clear();
		if (this.options.delayIn){
			this._inDelay = this.show.delay(this.options.delayIn, this);
		} else {
			this.show();
		}
	},

	_leave: function(){
		this._clear();
		if (this.options.delayOut){
			this._outDelay = this.hide.delay(this.options.delayOut, this);
		} else {
			this.hide();
		}
	},

	_complete: function(){
		if (!this.visible){
			this.tip.dispose();
		}
		this.fireEvent('complete', this.visible);
	}

});